@@ -190,6 +190,8 @@ UI-specific implementation
 To complete the implementation of the app tours, the UI-specific framework has to provide a tour layout class, which is
 highlighting the widget explained and to display a tooltip and the tour page texts.
 
+.. hint:: the :class:`~ae.kivy_help.TourOverlay` class is a quite complete implementation of a tour layout class.
+
 Optionally for the 'user_registration' page of the :class:`OnboardingTour` a open user name editor flow has to be
 implemented.
 """
@@ -203,7 +205,7 @@ from ae.gui_app import (
     FLOW_KEY_SEP, flow_action, flow_class_name, flow_path_id, id_of_flow, update_tap_kwargs, MainAppBase)
 
 
-__version__ = '0.1.30'
+__version__ = '0.1.31'
 
 
 register_package_translations()
@@ -314,7 +316,9 @@ def id_of_tour_help(page_id: str) -> str:
     return f'{TOUR_PAGE_HELP_ID_PREFIX}{page_id}'
 
 
-def layout_ps_hints(wid_x: float, wid_y, wid_width, wid_height, win_width, win_height) -> PsHintsType:
+def layout_ps_hints(wid_x: float, wid_y: float,
+                    wid_width: float, wid_height: float,
+                    win_width: float, win_height: float) -> PsHintsType:
     """ recalculate anchor and max width/height on change of widget pos/size or window size.
 
     :param wid_x:           x coordinate in app window of help target flow widget.
@@ -688,8 +692,13 @@ class HelpAppBase(MainAppBase, ABC):
         if self.user_id not in self.registered_users:
             self.start_app_tour()
 
-    def on_flow_popup_close(self, _flow_key: str, _popup_args: Dict[str, Any]) -> bool:
-        """ overwritten popup close handler of FlowPopup widget to reset help widget/text. """
+    def on_flow_popup_close(self, _flow_key: str, _event_kwargs: Dict[str, Any]) -> bool:
+        """ overwritten popup close handler of FlowPopup widget to reset help widget/text.
+
+        :param _flow_key:       (unused)
+        :param _event_kwargs:   (unused)
+        :return:                always True.
+        """
         if self.help_layout and self.help_widget(self.displayed_help_id, dict()) is self.help_activator:
             self.help_display('', dict())
         return True
@@ -777,7 +786,7 @@ class TourBase:
 
     def __init__(self, main_app: HelpAppBase):
         main_app.vpo(f"TourBase.__init__(): tour overlay={main_app.tour_layout}")
-        self.auto_switch_next_page_request = None
+        self._auto_switch_next_page_request = None
         self._delayed_setup_layout_call = None
         self._saved_app_states: Dict[str, Any] = dict()
 
@@ -793,9 +802,9 @@ class TourBase:
 
     def auto_switch_cancel(self):
         """ cancel auto switch callback if requested. """
-        if self.auto_switch_next_page_request:
-            self.auto_switch_next_page_request.cancel()
-            self.auto_switch_next_page_request = None
+        if self._auto_switch_next_page_request:
+            self._auto_switch_next_page_request.cancel()
+            self._auto_switch_next_page_request = None
         self.auto_switch_pages = False
 
     def auto_switch_next_page(self):
@@ -804,7 +813,7 @@ class TourBase:
         delay = self.page_data.get('next_page_delay', 9.6)
         main_app.vpo(f"TourOverlay.auto_switch_next_page from page #{self.page_idx} delay={delay}")
         if self.page_idx + 1 < len(self.page_ids):
-            self.auto_switch_next_page_request = main_app.call_method_delayed(delay, self.next_page)
+            self._auto_switch_next_page_request = main_app.call_method_delayed(delay, self.next_page)
 
     def backup_app_states(self):
         """ backup current states of this app, including flow. """
@@ -841,7 +850,6 @@ class TourBase:
     def next_page(self):
         """ switch to next tour page. """
         self.teardown_page()
-        self.setup_next_page()
 
         ids = self.page_ids
         assert self.page_idx + 1 < len(ids), f"TourBase.next_page missing {self.__class__.__name__}:{self.page_idx + 1}"
@@ -856,8 +864,6 @@ class TourBase:
         self.auto_switch_cancel()
         self.teardown_page()
 
-        self.setup_prev_page()
-
         ids = self.page_ids
         assert self.page_idx > 0, f"TourBase.prev_page wrong/missing page {self.__class__.__name__}:{self.page_idx - 1}"
         self.last_page_idx = self.page_idx
@@ -925,10 +931,6 @@ class TourBase:
         if self.auto_switch_pages:
             self.auto_switch_next_page()
 
-    def setup_next_page(self):
-        """ prepare app flow to display next page. """
-        self.main_app.vpo("TourBase.setup_next_page")
-
     def setup_page(self):
         """ setup tour overlay to load data for to display new tour page. """
         self.main_app.vpo(f"TourBase.setup_page page_data={self.page_data}")
@@ -938,10 +940,6 @@ class TourBase:
         app_flow_delay = self.page_data.get('app_flow_delay', 1.2)  # > 0.9 to complete auto width animation
         self._delayed_setup_layout_call = self.main_app.call_method_delayed(app_flow_delay, self.setup_layout)
 
-    def setup_prev_page(self):
-        """ setup app flow to display the previous tour page. """
-        self.main_app.vpo("TourBase.setup_prev_page")
-
     def setup_texts(self):
         """ setup texts in tour layout from page_data. """
         main_app = self.main_app
@@ -985,11 +983,11 @@ class TourBase:
             self._delayed_setup_layout_call = None
         self.auto_switch_cancel()
         self.teardown_page()
-        self.restore_app_states()
 
     def teardown_page(self):
         """ restore tour page state before switch to prev/next page or tour finishing. """
         self.main_app.vpo("TourBase.teardown_page")
+        self.restore_app_states()
 
     def update_page_ids(self):
         """ update/change page ids after app flow setup and before the tour overlay/layout setup.
@@ -1005,19 +1003,16 @@ class TourDropdownFromButton(TourBase):
     """ generic tour base class for to auto-explain a dropdown menu, starting with the button opening the dropdown. """
     determine_page_ids = '_v_'
 
-    def setup_next_page(self):
-        """ overwrite to prevent opening of sub-menus. """
-        super().setup_next_page()
-        if self.page_idx == 0:
-            this_page_id = self.page_ids[self.page_idx]
-            if flow_action(this_page_id) == 'open':
-                self.main_app.change_flow(this_page_id, **update_tap_kwargs(self.layout.explained_widget))
+    def setup_page(self):
+        """ prevent opening of sub-menus and close of dropdown if dropdown child would open another dropdown. """
+        super().setup_page()
+        if self.last_page_idx == 0 and self.page_idx == 1:
+            last_page_id = self.page_ids[self.last_page_idx]
+            if flow_action(last_page_id) == 'open':
+                self.main_app.change_flow(last_page_id, **update_tap_kwargs(self.layout.explained_widget))
 
-    def setup_prev_page(self):
-        """ overwrite to prevent close of dropdown if dropdown child would open another dropdown. """
-        super().setup_prev_page()
-        if self.page_idx == 1:
-            if flow_action(self.page_ids[self.page_idx - 1]) == 'open':
+        elif self.last_page_idx == 1 and self.page_idx == 0:
+            if flow_action(self.page_ids[self.page_idx]) == 'open':
                 self.main_app.popups_opened()[-1].close()
 
     def update_page_ids(self):
